<?php
namespace Neuron\Person;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;

class Attribute extends \Neuron\Generic\Entity\Mapper {

    protected $_storage;

    public function __construct(Attribute\Storage\Skeleton $storage,$data = []) {

        /* simpan storage */
        $this->_storage = $storage;

        /* map properti ke tabel */
        parent::__construct(
            $data,
            array(
                new Map("person_attribute_id"),
                new Map("attr_code"),
                new Map("person_id"),
                new Map("attr_value"),
                new Map("start_date"),
                new Map("end_date"),
                new Map("document_id"),
                new Map("update_dtm"),
                new Map("update_by"),
            ),
            parent::CASE_INSENSITIVE
        );
    }

    public function load($id, $mode = Attribute\Storage::LOADBY_ID) {

        /* query */
        if ($data = $this->_storage->load($id, $mode)) {
            
            $this->push($data);

            /* berhasil */
            return $this->person_attribute_id;
        }

        /* default gagal */
        return false;
    }

    public function save($update = false) {

        /* simpan */
        if ($id = $this->_storage->save($this, $update)) {

            /* set id */
            $this->person_attribute_id = $id;
            return $id;
        }

        /* return hasil */
        return false;
    }

    public function saveEx($rows,$files, \Neuron\Person\Entity $person, $sessionOwner){
        $result     = new Result();
        $attributes = new Attributes($this->_storage);

        foreach($rows as $row){
            $attrValue      = $row['value'];
            $attrCode       = $row['name'];
            $evidenceId     = null;
            /* Cari filenya dulu */
            foreach($files as $key => $file){
                if($key == $attrCode){
                    $evidence       = $this->saveAttributeFile($file,$person->id,$key,$sessionOwner);
                    $evidenceId     = $evidence->data;
                }
            }
            
            /* Cari dulu employee ini udah punya attribute dengan code sama atau belum, kalo belum*/
            $attribute  = new Attribute($this->_storage);
            $attribute->load(['person_id' => $person->id,'attr_code' => $attrCode],Attribute\Storage::LOADBY_PERSONID_ATTR_CODE);
            if($attribute->person_attribute_id){
                $attribute->attr_value   = $attrValue;
                $attribute->document_id  = $evidenceId ?: $attribute->document_id;
                $attribute->update_dtm   = date("Y-m-d H:i:s");
                $save = $attribute->save(true);
            }else{
                $attribute->push([
                    "attr_code" => $attrCode,
                    "person_id" => $person->id,
                    "attr_value" => $attrValue,
                    "start_date" => date("Y-m-d"),
                    "document_id" => $evidenceId,
                    "update_dtm" => date("Y-m-d H:i:s"),
                    "update_by" => $sessionOwner,
                ]);     
                $save = $attribute->save(false);
            }
            $attributes->add($attribute);
        }
        
        $result->data   = $attributes;
        return $result;
    }

    public function saveAttributeFile($file,$attributeOwner,$attr_code,$sessOwner){
        $result     = new Result;
        // Simpan attribute files to document
        $uploadConfig 	= $this->_storage->getConfig()['upload_document'];
        $uploadPath = $uploadConfig['location'];
        if($uploadConfig['split_folder']){
            $uploadPath = $uploadPath . '/' . date($uploadConfig['split_folder']);
        }

        $upload 		= new \Neuron\Upload\Upload(
            'uploader', $this->_storage->getConfig()['upload_document']
        );

        $upload->handle(array('file' => $file));
        $file_name 	        = $upload->name;
        $file_size 	        = $upload->size;
        $file_type 	        = pathinfo($upload->name, PATHINFO_EXTENSION);
        $file_dir 	        = $uploadPath;

        $documentType 		= new \Neuron\Document\Type(
            \Neuron\Document\Type\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            )
        );
        $dataDocumentType 		= $documentType->loadByName($attr_code);
        $documentTypeId 		= $dataDocumentType['document_type_id'];

        $document 			= new \Neuron\Document\Entity(
            \Neuron\Document\Entity\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            )
        );
        
        $dataDocument 		= array(
            "document_no"		=>	"EMP/ATTR/".$attr_code."/".$attributeOwner.'/'.date('Ymd'),
            "document_name"		=>	"Employee Attribute File (".$attr_code.")",
            "document_desc"		=>	"Employee Attribute File (".$attr_code.")",
            "document_dtm"		=>	date('Y-m-d H:i:s'),
            "document_type_id"	=>	$documentTypeId,
            "document_by"		=>	$sessOwner,
            "document_end_dtm"	=> 	null,
        );

        $document->push($dataDocument);
        if($documentId  = $document->save(false)){
            $documentFiles 	= new \Neuron\Document\Files(
                \Neuron\Document\Files\Storage::factory(
                    $this->getDb(),
                    $this->getConfig()
                )
            );
            $dataFiles 		= array(
                "document_id" => $document->document_id,
                "file_name" => $file_name,
                "file_dir" => $file_dir,
                "file_type" => $file_type,
                "file_size" => $file_size,
                "document_file_status" => 1,
            );

            $documentFiles->push($dataFiles);
            $saveFile = $documentFiles->save(false);
            if($saveFile){
                $documentOwner 	= new \Neuron\Document\Owner(
                    \Neuron\Document\Owner\Storage::factory(
                        $this->getDb(),
                        $this->getConfig()
                    )
                );
                $ownerData 		= array(
                    "document_id"	 	=> $document->document_id,
                    "owner_ref_code" 	=> "EMPLOYEE",
                    "owner_ref_id" 		=> $attributeOwner,
                    "domain_name" 		=> null,
                    "domain_id" 		=> null,
                );
                $documentOwner->push($ownerData);
                if($documentOwner->save(false)){	
                    $audit 	= new \Neuron\Document\Audit(
                        \Neuron\Document\Audit\Storage::factory(
                            $this->getDb(),
                            $this->getConfig()
                        )
                    );
                    $dataAudit 	= array(
                        "document_id" => $document->document_id,
                        "user_id" => $sessOwner,
                        "audit_type_id" => 1,
                        "audit_dtm" => date('Y-m-d'),
                        "notes" => null
                    );
                }
            }
        }

        $result->data 	= $documentId;
        return $result;
    }

}
