<?php

namespace Neuron\Person;

use Neuron\Generic\Result;
use Neuron\Generic\Set;
use Neuron\Person\Medical;

class Medicals extends Set {
    
	protected $_storage;

	public function __construct(\Neuron\Person\Medical\Storage\Skeleton $storage) {
	  
		$this->_storage = $storage;
	}

  	public function listdata($params) {

		try {
            $result = $this->_storage->listdata($params);
            
            $retval = array(
                'draw' =>  isset($params['draw'])? $params['draw'] : 0,
                'recordsTotal' => $result['total'],
                'recordsFiltered' => $result['total'],
                'data' => $result['data'],
            );

            return $retval;
        } catch (Exception $e) {
            
        }
	}
    

    public function load($ids = '', $mode = Medical\Storage::LIST_ALL, $params = array()){
        $result = new Result();
        try {
            /* clear list */
            $this->clear();

            if ($rs = $this->_storage->loadList($ids, $mode, $params)) {
                if ($rows = $rs->list) {
                    $rows = $rs->list;
                    $list = [];
                    foreach ($rows as $row) {
                        /* for issue oracle */
                        $row = array_change_key_case($row);
                        unset($row['b_rownum']);
                        /* new user */
                        $model = new Medical($this->_storage);
                        $model->push($row);
                        $this->add($model);

                        $parameters = new \Neuron\Person\Parameters(\Neuron\Person\Parameter\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig()));
                        $dataType = $parameters->loadBY(['param_ref_name'=> 'MEDICAL_TYPE', 'param_ref_id'=> $row['medical_type_id']]);
                        $row['medical_type_name'] = $dataType['value1'];
                        
                        $list[] = $row;
                    }

                    $result->data = [
                        'pager' => $rs->pager,
                        'list' => $list,
                        'countTotal' => $rs->countTotal,
                    ];
                } else {
                    $result->data = [];
                    $result->code = 2;
                    $result->info = 'load_none';
                }
            } else {
                $result->data = [];
                $result->code = 1;
                $result->info = 'load_not_found';
            }

            return $result;
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            $result->code = 500;
            $result->info = 'error';
            return $result;
        }
    }
}